%define nginx_user nginx
%define nginx_group nginx
%define epoch 1

%define __arch_install_post   /usr/lib/rpm/check-rpaths   /usr/lib/rpm/check-buildroot

%if 0%{?rhel}
%define _group System Environment/Daemons
%endif

%if (0%{?el7}) && (0%{?amzn} == 0)
Epoch: %{epoch}
Requires(pre): shadow-utils
Requires: openssl >= 1.0.2
Requires: procps-ng
BuildRequires: openssl-devel >= 1.0.2
%define dist .el7
%endif

%if (0%{?el7}) && (0%{?amzn} == 2)
Epoch: %{epoch}
Requires(pre): shadow-utils
Requires: openssl11 >= 1.1.1
Requires: procps-ng
BuildRequires: openssl11-devel >= 1.1.1
%endif

%if 0%{?el8}
Epoch: %{epoch}
Requires(pre): shadow-utils
Requires: procps-ng
BuildRequires: openssl3-devel
%define _debugsource_template %{nil}
%endif

%if 0%{?el9}
Epoch: %{epoch}
Requires(pre): shadow-utils
Requires: procps-ng
BuildRequires: openssl-devel
%define _debugsource_template %{nil}
%endif

%if 0%{?suse_version} >= 1315
%define _group Productivity/Networking/Web/Servers
%define nginx_loggroup trusted
Requires(pre): shadow
Requires: procps
BuildRequires: libopenssl-devel
%define _debugsource_template %{nil}
%endif

# This also applies to Amazon Linux 2023
%if 0%{?fedora}
%define _debugsource_template %{nil}
%global _hardened_build 1
%define _group System Environment/Daemons
Requires: procps-ng
BuildRequires: openssl-devel
Requires(pre): shadow-utils
%endif

BuildRequires: libedit-devel
BuildRequires: libxml2-devel
BuildRequires: libxslt-devel
BuildRequires: which
%if ((0%{?rhel} >= 8) || (0%{?amzn} >= 2023))
BuildRequires: libatomic
%endif

%define base_version 1.27.4
%define base_release 1%{?dist}
%define njs_version 0.8.8
%define quickjs_version 2024.07.28

%define bdir %{_builddir}/%{name}-%{base_version}

Summary: freenginx njs dynamic modules
Name: freenginx-module-njs
Version: %{base_version}+%{njs_version}
Release: %{base_release}
URL: https://freenginx.org/
Group: %{_group}

Source0: https://freenginx.org/download/freenginx-%{base_version}.tar.gz
Source1: %{name}.copyright

Source100: njs-%{njs_version}.tar.gz
Source101: quickjs-%{quickjs_version}.tar.xz

Patch100: quickjs_quickjs.makefile.patch

License: 2-clause BSD-like license

BuildRoot: %{_tmppath}/%{name}-%{base_version}-%{base_release}-root
BuildRequires: zlib-devel
BuildRequires: pcre2-devel
Requires: freenginx-%{epoch}:%{base_version}
Provides: %{name}-%{epoch}:%{version}
Conflicts: nginx-module-njs

%description
freenginx njs dynamic modules.

%if 0%{?suse_version}
%debug_package
%endif

%define WITH_CC_OPT $(echo %{optflags} $(pcre2-config --cflags))
%define WITH_LD_OPT -Wl,-z,relro -Wl,-z,now -Wl,-as-needed

%define BASE_CONFIGURE_ARGS $(echo "--prefix=%{_sysconfdir}/nginx --sbin-path=%{_sbindir}/nginx --modules-path=%{_libdir}/nginx/modules --conf-path=%{_sysconfdir}/nginx/nginx.conf --error-log-path=%{_localstatedir}/log/nginx/error.log --http-log-path=%{_localstatedir}/log/nginx/access.log --pid-path=%{_localstatedir}/run/nginx.pid --lock-path=%{_localstatedir}/run/nginx.lock --http-client-body-temp-path=%{_localstatedir}/cache/nginx/client_temp --http-proxy-temp-path=%{_localstatedir}/cache/nginx/proxy_temp --http-fastcgi-temp-path=%{_localstatedir}/cache/nginx/fastcgi_temp --http-uwsgi-temp-path=%{_localstatedir}/cache/nginx/uwsgi_temp --http-scgi-temp-path=%{_localstatedir}/cache/nginx/scgi_temp --user=%{nginx_user} --group=%{nginx_group} --with-compat --with-file-aio --with-threads --with-http_addition_module --with-http_auth_request_module --with-http_dav_module --with-http_flv_module --with-http_degradation_module --with-http_gunzip_module --with-http_gzip_static_module --with-http_mp4_module --with-http_random_index_module --with-http_realip_module --with-http_secure_link_module --with-http_slice_module --with-http_ssl_module --with-http_stub_status_module --with-http_sub_module --with-http_v2_module $( if [ 0%{?rhel} -eq 7 ] || [ 0%{?suse_version} -eq 1315 ]; then continue; else echo "--with-http_v3_module"; fi; ) --with-mail --with-mail_ssl_module --with-stream --with-stream_realip_module --with-stream_ssl_module --with-stream_ssl_preread_module")
%define MODULE_CONFIGURE_ARGS $(echo "--add-dynamic-module=njs-%{njs_version}/nginx")

%prep
%setup -qcTn %{name}-%{base_version}
tar --strip-components=1 -zxf %{SOURCE0}

ln -s . nginx%{?base_suffix}

%if 0%{?el8}
sed \
  -e 's|\(ngx_feature_path=\)$|\1%{_includedir}/openssl3|' \
  -e 's|\(ngx_feature_libs="\)|\1-L%{_libdir}/openssl3 |' \
  -i auto/lib/openssl/conf
%endif

tar xvzfo %{SOURCE100}
ln -s njs-* njs
tar xvJfo %{SOURCE101}
ln -s quickjs-* quickjs

cd %{bdir}/quickjs
%patch100 -p1

%build
_nproc=`getconf _NPROCESSORS_ONLN`
if [ $_nproc -gt 1 ]; then
    _make_opts="-j$_nproc"
fi

%define __with_quickjs %{nil}
%if (0%{?rhel} >= 8) || (0%{?amzn} >= 2023) || (0%{?suse_version} >= 1500)
cd %{bdir}/quickjs-%{quickjs_version} && make $_make_opts install PREFIX=%{bdir}/prebuilt/
%define __with_quickjs --with-quickjs
%endif
cd %{bdir}/njs-%{njs_version} && sed -i.bak 's,-lquickjs.lto ,-lquickjs.lto -latomic ,g' auto/quickjs && sed -i.bak 's,-lquickjs ,-lquickjs -latomic ,g' auto/quickjs && ./configure %{__with_quickjs} --cc-opt="%{WITH_CC_OPT} -I%{bdir}/prebuilt/include/quickjs/" --ld-opt="%{WITH_LD_OPT} -L%{bdir}/prebuilt/lib/quickjs/" && make $_make_opts njs && mv build build-cli
cd %{bdir}

./configure %{BASE_CONFIGURE_ARGS} %{MODULE_CONFIGURE_ARGS} \
	--with-cc-opt="%{WITH_CC_OPT} " \
	--with-ld-opt="%{WITH_LD_OPT} " \
	--with-debug
make %{?_smp_mflags} modules
for so in `find %{bdir}/objs/ -type f -name "*.so"`; do
debugso=`echo $so | sed -e 's|\.so$|-debug.so|'`
mv $so $debugso
done

./configure %{BASE_CONFIGURE_ARGS} %{MODULE_CONFIGURE_ARGS} \
	--with-cc-opt="%{WITH_CC_OPT} " \
	--with-ld-opt="%{WITH_LD_OPT} "
make %{?_smp_mflags} modules

%install
cd %{bdir}
%{__rm} -rf %{buildroot}
%{__mkdir} -p %{buildroot}%{_datadir}/doc/%{name}
%{__install} -m 644 -p %{SOURCE1} %{buildroot}%{_datadir}/doc/%{name}/COPYRIGHT

%{__install} -m644 %{bdir}/njs-%{njs_version}/CHANGES %{buildroot}%{_datadir}/doc/%{name}/
%{__mkdir} -p %{buildroot}%{_bindir}
%{__install} -m755 %{bdir}/njs-%{njs_version}/build-cli/njs %{buildroot}%{_bindir}/

%{__mkdir} -p %{buildroot}%{_libdir}/nginx/modules
for so in `find %{bdir}/objs/ -maxdepth 1 -type f -name "*.so"`; do
%{__install} -m755 $so %{buildroot}%{_libdir}/nginx/modules/
done

%check
%{__rm} -rf %{buildroot}/usr/src
cd %{bdir}
grep -v 'usr/src' debugfiles.list > debugfiles.list.new && mv debugfiles.list.new debugfiles.list
cat /dev/null > debugsources.list
%if 0%{?suse_version} >= 1500
cat /dev/null > debugsourcefiles.list
%endif

%clean
%{__rm} -rf %{buildroot}

%files
%defattr(-,root,root)
%{_libdir}/nginx/modules/*
%dir %{_datadir}/doc/%{name}
%{_datadir}/doc/%{name}/*
%{_bindir}/njs

%post
if [ $1 -eq 1 ]; then
cat <<BANNER
----------------------------------------------------------------------

The njs dynamic modules for nginx have been installed.
To enable these modules, add the following to /etc/nginx/nginx.conf
and reload nginx:

    load_module modules/ngx_http_js_module.so;
    load_module modules/ngx_stream_js_module.so;

Please refer to the modules documentation for further details:
https://nginx.org/en/docs/njs/
https://nginx.org/en/docs/http/ngx_http_js_module.html
https://nginx.org/en/docs/stream/ngx_stream_js_module.html

----------------------------------------------------------------------
BANNER
fi

%changelog
* Wed Dec 18 2024 Teddysun <i@teddysun.com> - 1.27.4+0.8.8-1
- njs module updated to 0.8.8

* Tue Sep 10 2024 Teddysun <i@teddysun.com> - 1.27.4+0.8.7-1
- base version updated to 1.27.4

* Fri Aug 16 2024 Teddysun <i@teddysun.com> - 1.27.3+0.8.5-1
- base version updated to 1.27.3

* Tue Jul 16 2024 Teddysun <i@teddysun.com> - 1.27.2+0.8.5-2
- base version updated to 1.27.2
- njs module updated to 0.8.5
